#include "resource.h" // ID-Definitionen einbinden

#include <afxdlgs.h> // fr Standarddialoge
#include <stdlib.h> // nur fr Zufallszahlen 
#include "DemoMainFrame.h"

BEGIN_MESSAGE_MAP(DemoMainFrame, CFrameWnd)
	// Windows Messages:
	ON_WM_PAINT()
	ON_WM_CONTEXTMENU()

	// Notification Messages:
	ON_COMMAND(ID_FILE_OPEN, doOpen)
	ON_COMMAND(ID_FILE_SAVEAS, doSave)
	ON_COMMAND(ID_FILE_EXIT, doQuit)
	ON_COMMAND(ID_DRAW_RECT, doRect)
	// kompletten ID-Bereich abfangen:
	ON_COMMAND_RANGE(ID_DRAW_OVAL_HELLIPSE,
	                 ID_DRAW_OVAL_CIRCLE,
	                 doOval)
	ON_COMMAND(ID_VIEW_FILLED, doFill)
	ON_COMMAND(ID_VIEW_OUTLINE, doOutline)
	// aufrufen, bevor Men erscheint, damit wir
	// den aktuellen Eintrag markieren knnen:
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_OUTLINE,
	                           ID_VIEW_FILLED,
	                           doUpdateFillOutline)
	ON_UPDATE_COMMAND_UI_RANGE(ID_CONTEXT_RECT,
	                           ID_CONTEXT_CIRCLE,
	                           doUpdateContext)
	ON_COMMAND_RANGE(ID_CONTEXT_RECT, ID_CONTEXT_CIRCLE, doContext)
	END_MESSAGE_MAP()

DemoMainFrame::DemoMainFrame() :
isViewFilled(FALSE)
{
	Create(NULL, "Kontextmen verwenden!", WS_OVERLAPPEDWINDOW,
		   CRect(CPoint(100, 100), CSize(400, 400)),
		   NULL, MAKEINTRESOURCE(IDR_MAIN));
	// Men-Ressource verwenden; das Makro
	// MAKEINTRESOURCE (nur ein "S"!) verwandelt
	// Kennzahl in String

	myGraphicsList = new CObList();
	// per Zeiger, damit beim Dateiffnen einfach umzubiegen
}

DemoMainFrame::~DemoMainFrame()
{
	// Liste abrumen
	POSITION pos = myGraphicsList->GetHeadPosition();
	while(pos) // krzer fr while(p != NULL)
	{
		GraphicsObject* g = (GraphicsObject*) myGraphicsList->GetNext(pos);
		delete g;
	}

	delete myGraphicsList;
}

void DemoMainFrame::OnPaint()
{
	CPaintDC dc(this);
	
	POSITION pos = myGraphicsList->GetHeadPosition();
	// Liste durchlaufen und dabei zeichnen
	while(pos)
	{
		GraphicsObject* g = (GraphicsObject*) myGraphicsList->GetNext(pos);
		// auf nchstes stellen, alten Zeiger geben

		g->draw(&dc, isViewFilled);
	}
}

void DemoMainFrame::OnContextMenu(CWnd* pWnd, CPoint p)
{
	// p aus Bildschirm-Koordinaten
	// in Fensterkoordinaten verwandeln
	CPoint q(p); // kopieren, weil p noch bentigt
	ScreenToClient(&q);

	// Liste durchlaufen und nach Objekt
	// an den Koordinaten p suchen
	BOOL isFound = FALSE;
	POSITION pos = myGraphicsList->GetHeadPosition();
	GraphicsObject* g;
	while(pos)
	{
		g = (GraphicsObject*) myGraphicsList->GetNext(pos);
		if(g->isInside(q))
		{
			isFound = TRUE;
			break;
		}
	}
	
	if(!isFound) // kein Objekt unter Maus
	{
		return;
	}
	
	// jetzt ist also Objekt *g unter Maus
	recentObject = g;

	CMenu menu; 
    CMenu* contextMenu;
    menu.LoadMenu(IDR_CONTEXT);
	// nur das erste Men daraus
    contextMenu = menu.GetSubMenu(0);
	contextMenu->TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON,
	                            p.x, p.y, this, 0);
}

void DemoMainFrame::doOpen()
{
	char BASED_CODE szFilter[] = "Zeichnung (*.xyz)|*.xyz|Alle Daten (*.*)|*.*||";

	// Dateidialog ausfhren
	CFileDialog d(TRUE, "xyz", "Unbenannt", 0, szFilter); // TRUE = laden
	if( d.DoModal() != IDOK )
	{
		return; // Benutzer hat nicht auf OK geklickt
	}

	CFile file;
	int ok = file.Open((LPCTSTR)d.GetPathName(), CFile::modeRead);
	if(!ok)
	{
		MessageBox("Konnte Datei nicht zum Laden ffnen", "Fehler", MB_ICONERROR);
		return;
	}

	CObList* newGraphicsList = new CObList();

	try
	{
		CArchive ar(&file, CArchive::load);
		newGraphicsList->Serialize(ar);
		ar.Close();
		file.Close();
	}
	catch(CException* e)
	{	
		e->Delete(); // Exception war per Zeiger bergeben
		MessageBox("Fehler beim Laden", "Fehler", MB_ICONERROR);
		return;
	}

	// Wenn wir hier ankommen, hat das Laden geklappt:
	// also alte Daten lschen und Zeiger umbiegen

	POSITION pos = myGraphicsList->GetHeadPosition();
	while(pos) // krzer fr while(p != NULL)
	{
		GraphicsObject* g = (GraphicsObject*) myGraphicsList->GetNext(pos);
		delete g;
	}

	delete myGraphicsList;

	myGraphicsList = newGraphicsList;

	Invalidate();
}

void DemoMainFrame::doSave()
{
	char BASED_CODE szFilter[] = "Zeichnung (*.xyz)|*.xyz|Alle Daten (*.*)|*.*||";

	// Dateidialog ausfhren
	CFileDialog d(FALSE, "xyz", "Unbenannt", 0, szFilter); // FALSE = speichern
	if( d.DoModal() != IDOK )
	{
		return; // Benutzer hat nicht auf OK geklickt
	}

	CFile file;
	int ok = file.Open((LPCTSTR)d.GetPathName(),
	                      CFile::modeCreate | CFile::modeWrite);
	// modeCreate: berschreiben, wenn schon da

	if(!ok)
	{
		MessageBox("Konnte Datei nicht zum Speichern ffnen", "Fehler", MB_ICONERROR);
		return;
	}
	
	try
	{
		CArchive ar(&file, CArchive::store);
		myGraphicsList->Serialize(ar); // ruft Serialize der gelisteten Objekte auf
		ar.Close();
		file.Close();
	}
	catch(CException* e)
	{	
		e->Delete(); // Exception war per Zeiger bergeben
		MessageBox("Fehler beim Speichern", "Fehler", MB_ICONERROR);
		return;
	}
}

void DemoMainFrame::doQuit()
{
	this-> DestroyWindow();
}

// Rechteck anhngen
void DemoMainFrame::doRect()
{
	GraphicsObject* n = new GraphicsObject(GraphicsObject::RECT, rand()%300, rand()%300);
	if(n) // new funktioniert bei MS altertmlich (NULL statt Exception)
	{
		myGraphicsList->AddTail(n);
		Invalidate(); // neu zeichnen
	}
}

// Oval anhngen
void DemoMainFrame::doOval(UINT nID)
{
	// wandle bergebene Kennzahl in GraficsObjekt-Typ
	int type = -1;
	switch(nID)
	{
	case ID_DRAW_OVAL_VELLIPSE:
		type = GraphicsObject::VELLIPSE;
		break;
	case ID_DRAW_OVAL_HELLIPSE:
		type = GraphicsObject::HELLIPSE;
		break;
	case ID_DRAW_OVAL_CIRCLE:
		type = GraphicsObject::CIRCLE;
		break;
	}

	GraphicsObject* n = new GraphicsObject(type, rand()%300, rand()%300);
	if(n) // new funktioniert bei MS altertmlich (NULL statt Exception)
	{
		myGraphicsList->AddTail(n);
		Invalidate(); // neu zeichnen
	}
}

void DemoMainFrame::doFill()
{
	isViewFilled = TRUE;
	Invalidate(); // Darstellung anpassen
}

void DemoMainFrame::doOutline()
{
	isViewFilled = FALSE;
	Invalidate();
}

void DemoMainFrame::doUpdateFillOutline(CCmdUI *pCmdUI)
// wird vor ffnen fr jeden angegebenen Eintrag einzeln aufgerufen
{
	if(pCmdUI->m_nID == ID_VIEW_OUTLINE)
	{
		pCmdUI->SetRadio(!isViewFilled);
	}
	else if(pCmdUI->m_nID == ID_VIEW_FILLED)
	{
		pCmdUI->SetRadio(isViewFilled);
	}
}

void DemoMainFrame::doUpdateContext(CCmdUI *pCmdUI)
// wird vor ffnen fr jeden angegebenen Eintrag einzeln aufgerufen
{
	// schalte den Meineintrag des aktuellen Typs grau,
	// die anderen "enabled"
	int t = recentObject->getType();
	if(    (t == GraphicsObject::RECT) && (pCmdUI->m_nID == ID_CONTEXT_RECT)
	    || (t == GraphicsObject::VELLIPSE) && (pCmdUI->m_nID == ID_CONTEXT_VELLIPSE)
	    || (t == GraphicsObject::HELLIPSE) && (pCmdUI->m_nID == ID_CONTEXT_HELLIPSE)
	    || (t == GraphicsObject::CIRCLE) && (pCmdUI->m_nID == ID_CONTEXT_CIRCLE)
	)
	{
		pCmdUI->Enable(FALSE);
	}
	else
	{
		pCmdUI->Enable(TRUE);
	}
}

void DemoMainFrame::doContext(UINT nID)
{
	switch(nID)
	{
	case ID_CONTEXT_RECT:
		recentObject -> setType(GraphicsObject::RECT);
		break;
	case ID_CONTEXT_VELLIPSE:
		recentObject -> setType(GraphicsObject::VELLIPSE);
		break;
	case ID_CONTEXT_HELLIPSE:
		recentObject -> setType(GraphicsObject::HELLIPSE);
		break;
	case ID_CONTEXT_CIRCLE:
		recentObject -> setType(GraphicsObject::CIRCLE);
		break;
	}

	Invalidate();
}